@@ -266,7 +266,7 @@ func NewHugoSites(cfg deps.DepsCfg) (*HugoSites, error) {
 			warpc.Options{
 				CompilationCacheDir: compilationCacheDir,
 				PoolSize:            1,
-				Memory:              256, // 256 MiB (4096 MiB Max)
+				Memory:              384, // 384 MiB (4096 MiB Max)
 				Infof:               logger.InfoCommand("webp").Logf,
 				Warnf:               logger.WarnCommand("webp").Logf,
 			},
@@ -1056,7 +1056,7 @@ func (s *siteRefLinker) refLink(ref string, source any, relative bool, outputFor
 		if outputFormat != "" {
 			o := target.OutputFormats().Get(outputFormat)
 
-			if o == nil {
+			if o.IsZero() {
 				s.logNotFound(refURL.Path, fmt.Sprintf("output format %q", outputFormat), p, pos)
 				return s.notFoundURL, nil
 			}
@@ -1610,7 +1610,7 @@ func (s *Site) renderAndWritePage(statCounter *uint64, targetPath string, p *pag
 	of := p.outputFormat()
 	p.incrRenderState()
 
-	ctx := tpl.Context.Page.Set(context.Background(), p)
+	ctx := s.TemplateStore.PrepareTopLevelRenderCtx(context.Background(), p)
 	ctx = tpl.Context.DependencyManagerScopedProvider.Set(ctx, p)
 
 	if err := s.renderForTemplate(ctx, p.Kind(), of.Name, d, renderBuffer, templ); err != nil {
@@ -1770,10 +1770,6 @@ func (s *Site) render(ctx *siteRenderContext) (err error) {
 		return
 	}
 
-	if err = s.renderMainLanguageRedirect(); err != nil {
-		return
-	}
-
 	return
 }
 
